# Copyright (C) 2009-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

# -*- coding: utf-8 -*-

from tg import flash, request, redirect, expose, tmpl_context, app_globals, validate
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.controllers.decorators import *

from luci.model import metadata, DBSession
from luci.model.objects import Node, Cluster, Task
from luci.model.auth import User

from luci.lib.ricci_helpers import send_batch_to_hosts
from luci.lib.ricci_communicator import RicciCommunicator
import luci.lib.ricci_queries as rq

import luci.lib.luci_tasks as luci_tasks

import logging
log = logging.getLogger(__name__)

def validate_create_cluster_form(self, **kw):
    errors = [];

    cluster_name = kw.get('cluster_name')
    if not cluster_name or len(cluster_name) < 1:
        flash(_('No cluster name was given'), 'error')
        return

    if len(cluster_name) > 15:
        flash(_('Cluster names must be less than 16 characters long'), 'error')
        return

    try:
        ret = int(DBSession.query(Cluster).filter_by(name=cluster_name).count())
        if ret != 0:
            flash(_('Luci is already managing a cluster named "%s"') % cluster_name, 'error')
            return
    except Exception, e:
        log.exception('Error querying the database while creating cluster %s' % cluster_name)
        flash(_('An error occurred while trying to query the luci database'), 'error')
        return

    create_opt_args = kw.get('opt')
    if create_opt_args is None:
        create_opts = []
    elif isinstance(create_opt_args, list):
        create_opts = create_opt_args
    else:
        create_opts = [ create_opt_args ]

    enable_storage = 'shared_storage' in create_opts
    reboot_nodes = 'reboot_nodes' in create_opts
    download_pkgs = kw.get('download_pkgs') == 'download'

    num_nodes = kw.get('num_nodes')
    if num_nodes:
        try:
            num_nodes = int(num_nodes) + 1
        except ValueError, e:
            num_nodes = None

    if not num_nodes:
        flash(_('No number of nodes was given'), 'error')
        return

    nodes = []
    for cur in xrange(num_nodes):
        cur_host = kw.get('hostname%d' % cur)
        cur_passwd = kw.get('password%d' % cur)
        cur_port = kw.get('port%d' % cur)
        if not cur_host:
            continue
        if not cur_port:
            cur_port = '11111'
        if not cur_passwd:
            errors.append(_('No password was provided for node "%s"') % cur_host)
            continue

        nodes.append([ cur_host, cur_passwd, cur_port ])

    if len(nodes) < 1:
        errors.append(_('No cluster nodes were given'))

    if len(errors) > 0:
        flash(_('The following errors occurred while creating cluster "%s": %s')
            % (cluster_name, ', '.join(errors)), 'error')
        return

    cluster_db_obj = Cluster(name=cluster_name, display_name=cluster_name)

    node_list = []
    node_db_obj = {}
    task_db_obj = {}
    for node in nodes:
        try:
            rc = RicciCommunicator(node[0], port=int(node[2]), enforce_trust=False)
            rc.trust()
            rc.auth(node[1])
            if not rc.authed():
                errors.append('Authentication to the ricci agent at %s:%d failed'
                    % (node[0], node[2]))
                continue

            rc = RicciCommunicator(node[0])
            cur_cluster_name = rc.cluster_info()[0]
            if cur_cluster_name:
                errors.append('%s is already a member of a cluster named "%s"'
                    % (node[0], cur_cluster_name))
                continue

            node_list.append(node[0])
            node_db_obj[node[0]] = Node(node_name=node[0],
                                        display_name=node[0],
                                        hostname=node[0],
                                        ipaddr=node[0],
                                        port=node[2])
            DBSession.add(node_db_obj[node[0]])
        except Exception, e:
            log.exception('Error adding node %s while creating cluster %s'
                % (node[0], cluster_name))
            errors.append(str(e))

    if len(errors) > 0:
        flash(_('The following errors occurred while creating cluster "%s": %s')
            % (cluster_name, ', '.join(errors)), 'error')
        DBSession.rollback()
        return

    ret = send_batch_to_hosts(node_list, 10, rq.create_cluster,
                'rhel6', cluster_name, cluster_name,
                node_list, True, True, enable_storage, False,
                download_pkgs, None, reboot_nodes)

    for i in ret.iterkeys():
        if ret[i].has_key('error'):
           errors.append(_('Unable to connect to the ricci agent on node %s: %s') % (i, ret[i]['err_msg']))
        task_id = ret[i]['batch_result'][0]
        task_obj = Task(batch_id=task_id,
                        task_type=luci_tasks.TASK_CLUSTER_CREATE,
                        batch_status=-1,
                        redirect_url='/clusters/%s/' % cluster_name,
                        status_msg=_('Creating node "%s" for cluster "%s"') % (i, cluster_name))
        DBSession.add(task_obj)
        task_db_obj[i] = task_obj

    try:
        cluster_db_obj.nodes = node_db_obj.values()
        cluster_db_obj.tasks = task_db_obj.values()
        for i in node_db_obj.iterkeys():
            node_db_obj[i].tasks = [ task_db_obj[i] ]
        DBSession.add(cluster_db_obj)
    except Exception, e:
        log.exception('Error updating luci DB for cluster %s' % cluster_name)
        DBSession.rollback()
        flash(_('An error occurred during the creation of cluster "%s" while updating the luci database: %s') % (cluster_name, str(e)), 'error')
        return

    flash(_('Creating the cluster "%s", this may take up to 30 seconds.  Please refresh screen to view created cluster.') % cluster_name, 'info')
    redirect("/cluster")
